using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using VRTK.Highlighters;

namespace VRTK
{
    public class DrawManager : MonoBehaviour
    {
        public Transform draw;
        public VRTK_ControllerEvents ce;
        public VRTK_DestinationMarker pointer;
        public GameObject controllerRef;
        
        public bool logSetEvent = true;
        public MenuScript ms;

        Transform current = null;
        Color currentStart = Color.gray;
        Color currentEnd = Color.gray; 

        public void onPress(object sender, ControllerInteractionEventArgs e)
        {
            if (!ms.editMode)
            {
                if (ms.menuOn) return;
                Instantiate(draw, Vector3.zero, Quaternion.identity, gameObject.transform);
            } else
            {
                if (ms.menuOn) return;
                if (current == null) return;
                if (e.touchpadAxis.y < 0.99f && e.touchpadAxis.y > 0f)
                {
                    current.GetComponent<LineRenderer>().ScaleUp(2);
                }
                if (e.touchpadAxis.y < 0f && e.touchpadAxis.y > -0.99f)
                {
                    current.GetComponent<LineRenderer>().ScaleDown(2);
                }
            }
            Debug.Log("press");
        }

        public void deleteCurrent()
        {
            if (ms.editMode)
            {
                if (current == null) return;
                Destroy(current.gameObject);
            }
        }

        public void deleteAll()
        {
            for (int i = 0; i < transform.childCount; ++i)
            {
                Destroy(transform.GetChild(i).gameObject);
            }
        }

        public void onTriggerPressed(object sender, ControllerInteractionEventArgs e)
        {
            controllerRef.GetComponent<VRTK_StraightPointerRenderer>().tracerVisibility = VRTK_BasePointerRenderer.VisibilityStates.OnWhenActive;
            controllerRef.GetComponent<VRTK_StraightPointerRenderer>().cursorVisibility = VRTK_BasePointerRenderer.VisibilityStates.OnWhenActive;
        }

        public void onTriggerReleased(object sender, ControllerInteractionEventArgs e)
        {
            controllerRef.GetComponent<VRTK_StraightPointerRenderer>().tracerVisibility = VRTK_BasePointerRenderer.VisibilityStates.AlwaysOff;
            controllerRef.GetComponent<VRTK_StraightPointerRenderer>().cursorVisibility = VRTK_BasePointerRenderer.VisibilityStates.AlwaysOff;
        }

        public void unselect()
        {
            if (current != null)
            {
                var currentLineRenderer = current.GetComponent<LineRenderer>();
                currentLineRenderer.startColor = currentStart;
                currentLineRenderer.endColor = currentEnd;
                current = null;
            }
        }

        void Start()
        {
            ce = controllerRef.GetComponent<VRTK_ControllerEvents>();
            pointer = controllerRef.GetComponent<VRTK_Pointer>();
            ce.TouchpadPressed += onPress;
            
            ce.TriggerPressed += onTriggerPressed;
            ce.TriggerReleased += onTriggerReleased;

            pointer.DestinationMarkerSet += DestinationMarkerSet;
        }

        void Update()
        {
            if (ms.editMode)
            {
                if (ce.triggerClicked && current != null)
                {
                    if (ms.menuOn) return;
                    current.GetComponent<LineRenderer>().Move(controllerRef.transform.position);
                }
            }
        }

        protected virtual void DestinationMarkerSet(object sender, DestinationMarkerEventArgs e)
        {
            if (e.target.tag != "drawing") return;
            LineRenderer currentLineRenderer;
            if (current != null)
            {
                currentLineRenderer = current.GetComponent<LineRenderer>();
                currentLineRenderer.startColor = currentStart;
                currentLineRenderer.endColor = currentEnd;
                if(current.Equals(e.target))
                {
                    ms.editMode = false;
                    current = null;
                    return;
                }
            }
            currentLineRenderer = e.target.GetComponent<LineRenderer>();
            if (!currentLineRenderer) return;
            currentStart = currentLineRenderer.startColor;
            currentLineRenderer.startColor = Color.magenta;
            currentEnd = currentLineRenderer.endColor;
            currentLineRenderer.endColor = Color.magenta;
            current = e.target;
            ms.editMode = true;
            if (logSetEvent)
            {
                DebugLogger(VRTK_ControllerReference.GetRealIndex(e.controllerReference), "POINTER SET", e.target, e.raycastHit, e.distance, e.destinationPosition);
            }
        }

        protected virtual void DebugLogger(uint index, string action, Transform target, RaycastHit raycastHit, float distance, Vector3 tipPosition)
        {
            string targetName = (target ? target.name : "<NO VALID TARGET>");
            string colliderName = (raycastHit.collider ? raycastHit.collider.name : "<NO VALID COLLIDER>");
            VRTK_Logger.Info("Controller on index '" + index + "' is " + action + " at a distance of " + distance + " on object named [" + targetName + "] on the collider named [" + colliderName + "] - the pointer tip position is/was: " + tipPosition);
        }

    }
}
