using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using VRTK.Highlighters;


namespace VRTK
{
    [RequireComponent(typeof(LineRenderer))]
    [RequireComponent(typeof(MeshCollider))]
    public class LineRenderer : MonoBehaviour
    {

        List<Vector3> linePoints = new List<Vector3>();
        LineRenderer lineRenderer;
        MeshCollider meshCollider;
        float startWidth = 0.035f;
        float endWidth = 0.035f;
        public float threshold = 0.001f;
        public VRTK_ControllerEvents ce;
        int lineCount = 0;
        bool shouldDraw = true;
        Vector3 avg = Vector3.zero;
        float scaleFactor = 1.2f;
        int scaleLimit = 0;
        Mesh initMesh;

        Vector3 lastPos = Vector3.one * float.MaxValue;

        public void onRelease(object sender, ControllerInteractionEventArgs e)
        {
            if (!lineRenderer) return;
            shouldDraw = false;
            Mesh m = new Mesh();
            lineRenderer.BakeMesh(m,true);
            meshCollider.sharedMesh = m;
            initMesh = m;

            ce.TouchpadReleased -= onRelease;
        }

        void Awake()
        {
            lineRenderer = GetComponent<LineRenderer>();
            lineRenderer.material = new Material(Shader.Find("Particles/Standard Unlit"));
            lineRenderer.startColor = Random.ColorHSV();
            lineRenderer.endColor = Random.ColorHSV();
            ce = transform.parent.GetComponent<DrawManager>().ce;
            meshCollider = GetComponent<MeshCollider>();
            ce.TouchpadReleased += onRelease;
        }

        void Update()
        {
            if (!ce.touchpadPressed)
            {
                
            }
            if (shouldDraw)
            {
                Vector3 drawingPos = ce.transform.position;


                float dist = Vector3.Distance(lastPos, drawingPos);
                if (dist <= threshold)
                    return;

                lastPos = drawingPos;
                if (linePoints == null)
                    linePoints = new List<Vector3>();
                linePoints.Add(drawingPos);
                avg = ((avg * (linePoints.Count - 1)) + drawingPos) / linePoints.Count;

                UpdateLine();
            }
        }


        void UpdateLine()
        {
            lineRenderer.startWidth = startWidth;
            lineRenderer.endWidth = endWidth;
            lineRenderer.positionCount = linePoints.Count;

            for (int i = lineCount; i < linePoints.Count; i++)
            {
                lineRenderer.SetPosition(i, linePoints[i]);
            }
            lineCount = linePoints.Count;
        }

        public void ScaleDown(int factor)
        {
            if (scaleLimit == -5) return;
            --scaleLimit;
            lineRenderer.startWidth /= scaleFactor;
            lineRenderer.endWidth /= scaleFactor;

            Mesh m = new Mesh();
            lineRenderer.BakeMesh(m, true);
            if (scaleLimit < 0) m = initMesh;
            meshCollider.sharedMesh = m;
        }

        public void ScaleUp(int factor)
        {
            if (scaleLimit == 5) return;
            ++scaleLimit;
            lineRenderer.startWidth *= scaleFactor;
            lineRenderer.endWidth *= scaleFactor;
            
            Mesh m = new Mesh();
            lineRenderer.BakeMesh(m, true);
            if (scaleLimit < 0) m = initMesh;
            meshCollider.sharedMesh = m;
        }

        public void Move(Vector3 point)
        {
            Vector3 sum = Vector3.zero;
            for (int i = 0; i < linePoints.Count; ++i)
            {
                linePoints[i] = (linePoints[i] + 0.15f*(point - avg));
                sum += linePoints[i];
                lineRenderer.SetPosition(i, linePoints[i]);
            }
            avg = sum/lineCount;
            Mesh m = new Mesh();
            lineRenderer.BakeMesh(m, true);
            meshCollider.sharedMesh = m;
        }
    }
}