using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace VRTK
{
    public class MenuScript : MonoBehaviour
    {
        public VRTK_ControllerEvents rightControllerEvents;
        public VRTK_ControllerEvents leftControllerEvents;
        public VRTK_DestinationMarker rightPointer;
        public VRTK_DestinationMarker leftPointer;

        public bool editMode = false;
        public bool menuOn = false;

        public DrawManager rightDrawManager;
        public DrawManager leftDrawManager;

        MeshRenderer deleteRenderer;
        BoxCollider deleteCollider;
        MeshRenderer clearRenderer;
        BoxCollider clearCollider;
        MeshRenderer howToPlayRenderer;
        BoxCollider howToPlayCollider;
        MeshRenderer quitRenderer;
        BoxCollider quitCollider;
        MeshRenderer backRenderer;
        BoxCollider backCollider;

        List<string> menuIdentifiers = new List<string> { "delete", "clear", /*"edit",*/ "how_to_play", "back", "quit" };
        List<string> drawingIdentifiers = new List<string> { "drawing" };

        Canvas howToCanvas;

        public void hideMenu(bool isMenuOn)
        {
            deleteRenderer.enabled = false;
            howToPlayRenderer.enabled = false;
            clearRenderer.enabled = false;
            quitRenderer.enabled = false;
            deleteCollider.enabled = false;
            howToPlayCollider.enabled = false;
            quitCollider.enabled = false;
            menuOn = isMenuOn;
            if (!menuOn)
            {
                rightPointer.targetListPolicy.identifiers = drawingIdentifiers;
                leftPointer.targetListPolicy.identifiers = drawingIdentifiers;
                disableBackButtonAndHowToCanvas();
            }
        }

        public void showMenuFromBack()
        {
            if (editMode)
            {
                deleteRenderer.enabled = true;
                deleteCollider.enabled = true;
            }
            clearRenderer.enabled = true;
            clearCollider.enabled = true;
            howToPlayRenderer.enabled = true;
            howToPlayCollider.enabled = true;
            quitRenderer.enabled = true;
            quitCollider.enabled = true;
        }
        public void enableBackButtonAndHowToCanvas()
        {
            backRenderer.enabled = true;
            backCollider.enabled = true;
            howToCanvas.enabled = true;
        }
        public void disableBackButtonAndHowToCanvas()
        {
            backRenderer.enabled = false;
            backCollider.enabled = false;
            howToCanvas.enabled = false;
        }

        private void OnEnable()
        {
            rightControllerEvents.ButtonTwoPressed += onButtonTwo;
            leftControllerEvents.ButtonTwoPressed += onButtonTwo;
        }
        void Start()
        {
            deleteRenderer = GameObject.FindGameObjectWithTag("delete").GetComponent<MeshRenderer>();
            deleteCollider = GameObject.FindGameObjectWithTag("delete").GetComponent<BoxCollider>();
            clearRenderer = GameObject.FindGameObjectWithTag("clear").GetComponent<MeshRenderer>();
            clearCollider = GameObject.FindGameObjectWithTag("clear").GetComponent<BoxCollider>();
            howToPlayRenderer = GameObject.FindGameObjectWithTag("how_to_play").GetComponent<MeshRenderer>();
            howToPlayCollider = GameObject.FindGameObjectWithTag("how_to_play").GetComponent<BoxCollider>();
            quitRenderer = GameObject.FindGameObjectWithTag("quit").GetComponent<MeshRenderer>();
            quitCollider = GameObject.FindGameObjectWithTag("quit").GetComponent<BoxCollider>();

            rightPointer.targetListPolicy.operation = VRTK_PolicyList.OperationTypes.Include;
            leftPointer.targetListPolicy.operation = VRTK_PolicyList.OperationTypes.Include;

            rightPointer.targetListPolicy.checkType = VRTK_PolicyList.CheckTypes.Tag;
            rightPointer.targetListPolicy.identifiers = drawingIdentifiers;

            leftPointer.targetListPolicy.checkType = VRTK_PolicyList.CheckTypes.Tag;
            leftPointer.targetListPolicy.identifiers = drawingIdentifiers;

            backRenderer = GameObject.FindGameObjectWithTag("back").GetComponent<MeshRenderer>();
            backCollider = GameObject.FindGameObjectWithTag("back").GetComponent<BoxCollider>();
            howToCanvas = GameObject.FindGameObjectWithTag("how_to_canvas").GetComponent<Canvas>();
            hideMenu(false); 
            backRenderer.enabled = false;
            backCollider.enabled = false;
            howToCanvas.enabled = false;
        }

        protected void onButtonTwo(object sender, ControllerInteractionEventArgs e)
        {
            menuOn = !menuOn;
            if (menuOn)
            {
                rightPointer.targetListPolicy.identifiers = menuIdentifiers;
                leftPointer.targetListPolicy.identifiers = menuIdentifiers;
                if (editMode)
                {
                    deleteRenderer.enabled = true;
                    deleteCollider.enabled = true;
                } else
                {
                    deleteCollider.enabled = false;
                }
                clearRenderer.enabled = true;
                clearCollider.enabled = true;
                howToPlayRenderer.enabled = true;
                howToPlayCollider.enabled = true;
                quitRenderer.enabled = true;
                quitCollider.enabled = true;
            } else
            {
                hideMenu(false);
            }
        }
    }
}