using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using VRTK.Highlighters;
using TMPro;

namespace VRTK
{
    public class TextScript : MonoBehaviour
    {
        public VRTK_DestinationMarker rightPointer;
        public VRTK_DestinationMarker leftPointer;
        public Color hoverColor = Color.cyan;
        public Color selectColor = Color.yellow;
        public MenuScript ms;

        MeshRenderer backRenderer;
        BoxCollider backCollider;
        MeshRenderer picassoRenderer;
        MeshRenderer startRenderer;
        BoxCollider startCollider;
        MeshRenderer howToPlayRenderer;
        BoxCollider howToPlayCollider;
        Canvas howToCanvas;

        protected virtual void OnEnable()
        {
            rightPointer.DestinationMarkerSet += DestinationMarkerSet;
            rightPointer.DestinationMarkerEnter += DestinationMarkerEnter;
            rightPointer.DestinationMarkerExit += DestinationMarkerExit;

            leftPointer.DestinationMarkerSet += DestinationMarkerSet;
            leftPointer.DestinationMarkerEnter += DestinationMarkerEnter;
            leftPointer.DestinationMarkerExit += DestinationMarkerExit;
        }

        private void Start()
        {
            backRenderer = GameObject.FindGameObjectWithTag("back").GetComponent<MeshRenderer>();
            backCollider = GameObject.FindGameObjectWithTag("back").GetComponent<BoxCollider>();
            howToPlayRenderer = GameObject.FindGameObjectWithTag("how_to_play").GetComponent<MeshRenderer>();
            howToPlayCollider = GameObject.FindGameObjectWithTag("how_to_play").GetComponent<BoxCollider>();
            howToCanvas = GameObject.FindGameObjectWithTag("how_to_canvas").GetComponent<Canvas>();
            if (!ms)
            {
                picassoRenderer = GameObject.FindGameObjectWithTag("picasso").GetComponent<MeshRenderer>();
                startRenderer = GameObject.FindGameObjectWithTag("start").GetComponent<MeshRenderer>();
                startCollider = GameObject.FindGameObjectWithTag("start").GetComponent<BoxCollider>();
            } else
            {
                howToPlayRenderer.enabled = false;
                howToPlayCollider.enabled = false;
            }
            backRenderer.enabled = false;
            backCollider.enabled = false;
            howToCanvas.enabled = false;
        }
        void hideStartMenu()
        {
            picassoRenderer.enabled = false;
            startRenderer.enabled = false;
            startCollider.enabled = false;
            howToPlayRenderer.enabled = false;
            howToPlayCollider.enabled = false;
        }
        void showStartMenu()
        {
            picassoRenderer.enabled = true;
            startRenderer.enabled = true;
            startCollider.enabled = true;
            howToPlayRenderer.enabled = true;
            howToPlayCollider.enabled = true;
        }

        protected virtual void DestinationMarkerExit(object sender, DestinationMarkerEventArgs e)
        {
            ToggleHighlight(e.target, Color.white);
        }
        protected virtual void DestinationMarkerEnter(object sender, DestinationMarkerEventArgs e)
        {
            ToggleHighlight(e.target, hoverColor);
        }
        protected virtual void DestinationMarkerSet(object sender, DestinationMarkerEventArgs e)
        {
            ToggleHighlight(e.target, selectColor);
            if (e.target.tag == "start")
            {
                SceneManager.LoadScene("DrawingScene_desert");
            }
            else if (e.target.tag == "clear")
            {
                ms.hideMenu(false);
                ms.editMode = false;
                ms.rightDrawManager.deleteAll();
                ms.leftDrawManager.deleteAll();
            }
            else if (e.target.tag == "how_to_play")
            {
                if (!ms)
                {
                    hideStartMenu();
                    howToCanvas.enabled = true;
                    backRenderer.enabled = true;
                    backCollider.enabled = true;
                }
                else
                {
                    ms.hideMenu(true);
                    ms.enableBackButtonAndHowToCanvas();
                }
            }
            else if (e.target.tag == "back")
            {
                if (!ms)
                {
                    howToCanvas.enabled = false;
                    backRenderer.enabled = false;
                    backCollider.enabled = false;
                    showStartMenu();
                }
                else
                {
                    ms.disableBackButtonAndHowToCanvas();
                    ms.showMenuFromBack();
                }
            }
            else if (e.target.tag == "delete")
            {
                if (!ms) return;
                ms.rightDrawManager.deleteCurrent();
                ms.leftDrawManager.deleteCurrent();
                ms.editMode = false;
                ms.hideMenu(false);
            }
            else if (e.target.tag == "quit")
            {
                Application.Quit();
            }
        }
        protected virtual void ToggleHighlight(Transform target, Color color)
        {
            var tmp = target.GetComponent<TextMeshPro>();
            if (!tmp) return;
            tmp.faceColor = color;
        }
    }
}